@@ -97,6 +97,9 @@ pub fn induce<O: Sync>(
         params.structure_penalty,
     );
 
+    if cfg!(feature = "verbose") {
+        eprintln!("COMPRESSION: starting score: {}", best_score)
+    }
     if params.aic.is_finite() {
         loop {
             let fragment_expr = {
@@ -109,6 +112,9 @@ pub fn induce<O: Sync>(
                     || dsl.propose_inventions(&rescored_frontiers, params.arity, tx),
                     || rx.iter().collect::<Vec<_>>(),
                 );
+                if cfg!(feature = "verbose") {
+                    eprintln!("COMPRESSION: proposed {} fragments", proposals.len())
+                }
                 let best_proposal = proposals
                     .into_par_iter()
                     .filter_map(|fragment_expr| {
@@ -128,10 +134,16 @@ pub fn induce<O: Sync>(
                     })
                     .max_by(|&(_, ref x), &(_, ref y)| x.partial_cmp(y).unwrap());
                 if best_proposal.is_none() {
+                    if cfg!(feature = "verbose") {
+                        eprintln!("COMPRESSION: no sufficient proposals")
+                    }
                     break;
                 }
                 let (new_dsl, new_score) = best_proposal.unwrap();
                 if new_score <= best_score {
+                    if cfg!(feature = "verbose") {
+                        eprintln!("COMPRESSION: score did not improve")
+                    }
                     break;
                 }
                 dsl = new_dsl;
@@ -140,6 +152,13 @@ pub fn induce<O: Sync>(
                 let (fragment_expr, _, log_prior) = dsl.invented.pop().unwrap();
                 let inv = proposals::defragment(fragment_expr.clone());
                 dsl.invent(inv, log_prior).expect("invalid invention");
+                if cfg!(feature = "verbose") {
+                    eprintln!(
+                        "COMPRESSION: score improved to {} with invention {}",
+                        best_score,
+                        dsl.display(&fragment_expr),
+                    )
+                }
                 fragment_expr
             };
             let i = dsl.invented.len() - 1;
@@ -881,23 +900,19 @@ mod proposals {
             return Box::new(iter::once(Fragment::Expression(expr.clone())));
         }
         let rst: Box<Iterator<Item = Fragment> + 'a> = match *expr {
-            Expression::Application(ref f, ref x) => {
-                if !toplevel {
-                    Box::new((0..arity + 1).flat_map(move |fa| {
-                        let fs = from_particular(f, fa, false);
-                        let xs: Vec<_> =
-                            from_particular(x, (arity as i32 - fa as i32) as u32, false).collect();
-                        fs.into_iter().zip(iter::repeat(xs)).flat_map(|(f, xs)| {
-                            xs.into_iter().map(move |x| {
-                                Fragment::Application(Box::new(f.clone()), Box::new(x))
-                            })
-                        })
-                    }))
-                } else {
-                    Box::new(iter::empty())
-                }
-            }
-            Expression::Abstraction(ref body) => Box::new(
+            Expression::Application(ref f, ref x) => Box::new((0..arity + 1).flat_map(move |fa| {
+                let xa = (arity as i32 - fa as i32) as u32;
+                from_particular(f, fa, false)
+                    .into_iter()
+                    .zip(iter::repeat(
+                        from_particular(x, xa, false).collect::<Vec<_>>(),
+                    ))
+                    .flat_map(|(f, xs)| {
+                        xs.into_iter()
+                            .map(move |x| Fragment::Application(Box::new(f.clone()), Box::new(x)))
+                    })
+            })),
+            Expression::Abstraction(ref body) if !toplevel => Box::new(
                 from_particular(body, arity, false).map(|e| Fragment::Abstraction(Box::new(e))),
             ),
             _ => Box::new(iter::empty()),
