@@ -173,7 +173,7 @@ impl Language {
         let dsl = self.clone();
         spawn(move || {
             let tx = tx.clone();
-            let termination_condition = &mut |expr, logprior| tx.send((expr, logprior)).is_err();
+            let termination_condition = |expr, logprior| tx.send((expr, logprior)).is_err();
             enumerator::run(&dsl, tp, termination_condition)
         });
         Box::new(rx.into_iter())
@@ -200,8 +200,8 @@ impl Language {
     /// let tasks = circuits::make_tasks(100);
     /// let ec_params = ECParams {
     ///     frontier_limit: 10,
-    ///     search_limit_timeout: Some(std::time::Duration::new(2, 0)),
-    ///     search_limit_description_length: None,
+    ///     search_limit_timeout: None,
+    ///     search_limit_description_length: Some(10.0),
     /// };
     /// let params = lambda::CompressionParams::default();
     ///
@@ -572,7 +572,7 @@ impl EC for Language {
     type Params = CompressionParams;
     fn enumerate<F>(&self, tp: TypeSchema, termination_condition: F)
     where
-        F: FnMut(Self::Expression, f64) -> bool,
+        F: Fn(Expression, f64) -> bool + Send + Sync,
     {
         enumerator::run(self, tp, termination_condition)
     }
